#
# Copyright (c) 2015-2021 University of Antwerp, Aloxy NV.
#
# This file is part of pyd7a.
# See https://github.com/Sub-IoT/pyd7a for further info.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# author: Christophe VG <contact@christophe.vg>
# class implementation of status meta-data

# D7ASP Status
#
# STATUS      1 byte      Slave D7ASP Status 9.2.9.1
#   b7    NLS     NWL Security Enabled
#   b6    MISSED  Always 0 for Responses. For Requests - set to 0 when the 
#                 D7ATP ACK_BITMAP is empty.
#   b5    RETRY   Always 0 for Responses. For Request - depends on the value of 
#                 the D7ATP Transaction ID bit in ACK_BITMAP before reception 
#                 of the current Request. When set to 1, a segment with the 
#                 same REQUEST_ID and FIFO_TOKEN has already been vectored to 
#                 upper layer.
#   b4-b3 RFU
#   b2-b0 STATE   Session State
# FIFO_TOKEN  1 byte      Value of the D7ATP Dialog ID
# REQUEST_ID  1 bytes     Value of the D7ATP Transaction ID
# RESPONSE_TO 1 byte      D7ATP Addressee Access Profile's Tc in Compressed 
#                         Time Format.
# ADDRESSEE   1/3/9 bytes D7ATP Addressee
import struct

from d7a.phy.channel_id import ChannelID
from d7a.support.schema   import Validatable, Types
from d7a.types.ct         import CT
from d7a.d7anp.addressee import Addressee
from d7a.phy.channel_header import ChannelHeader

class Status(Validatable):

  SCHEMA = [{
    "channel_id"    : Types.OBJECT(ChannelID),
    "rx_level"      : Types.BYTE(),
    "link_budget": Types.BYTE(),
    "target_rx_level": Types.BYTE(),
    "nls"        : Types.BOOLEAN(),
    "missed"     : Types.BOOLEAN(),
    "retry"      : Types.BOOLEAN(),
    "ucast"      : Types.BOOLEAN(),
    "fifo_token" : Types.BYTE(),
    "seq_nr"     : Types.BYTE(),
    "response_to": Types.OBJECT(CT),
    "addressee"  : Types.OBJECT(Addressee)
  }]

  def __init__(self, channel_id, rx_level, link_budget, target_rx_level, nls, missed, retry, unicast,
                     fifo_token, seq_nr, response_to, addressee):
    self.channel_id = channel_id
    self.rx_level    = rx_level
    self.link_budget = link_budget
    self.target_rx_level = target_rx_level
    self.nls         = nls
    self.missed      = missed
    self.retry       = retry
    self.unicast       = unicast
    self.fifo_token  = fifo_token
    self.seq_nr  = seq_nr
    self.response_to = response_to
    self.addressee   = addressee
    super(Status, self).__init__()


  @staticmethod
  def parse(s):
    channel_id  = ChannelID.parse(s)
    rx_level        = s.read("int:8")
    link_budget     = s.read("uint:8")
    target_rx_level = s.read("uint:8")
    nls             = s.read("bool")
    missed          = s.read("bool")
    retry           = s.read("bool")
    unicast         = s.read("bool" )
    _               = s.read("pad:4")
    fifo_token      = s.read("uint:8")
    seq_nr          = s.read("uint:8")
    response_to     = CT.parse(s)
    addressee       = Addressee.parse(s)

    return Status(channel_id=channel_id,
                    rx_level=rx_level, link_budget=link_budget,
                    target_rx_level=target_rx_level, nls=nls, missed=missed,
                    retry=retry, unicast=unicast, fifo_token=fifo_token,
                    seq_nr=seq_nr, response_to=response_to, addressee=addressee)

  def __iter__(self):
    for byte in self.channel_id: yield byte
    yield self.rx_level
    yield self.link_budget
    yield self.target_rx_level
    byte = 0
    if self.nls:    byte |= 1 << 7
    if self.missed: byte |= 1 << 6
    if self.retry:  byte |= 1 << 5
    if self.unicast:  byte |= 1 << 4
    yield byte
    yield self.fifo_token
    yield self.seq_nr
    for byte in self.response_to: yield byte
    for byte in self.addressee: yield byte

  def __str__(self):
    return "unicast={}, nls={}, retry={}, missed={}, fifo_token={}, rx_level={}, seq_nr={}, target_rx_level={}, " \
           "addressee={}, response_to={}, link_budget={}, channel_id={}".format(
      self.unicast,
      self.nls,
      self.retry,
      self.missed,
      self.fifo_token,
      self.rx_level,
      self.seq_nr,
      self.target_rx_level,
      self.addressee,
      self.response_to,
      self.link_budget,
      self.channel_id
    )